// See the end of this file for copyright and license terms.

use std::str::Chars;

#[derive(Clone)]
pub(crate) struct Cursor<'a> {
    stream: Chars<'a>,
    chop_start: String,
    line: usize,
    col: usize,
}

impl<'a> Cursor<'a> {
    pub const fn new(stream: Chars<'a>) -> Cursor<'a> {
        Cursor {
            stream,
            chop_start: String::new(),
            line: 1,
            col: 1,
        }
    }

    pub fn next(&mut self) -> Option<char> {
        let c = self.stream.next()?;
        self.chop_start.push(c);

        self.col += 1;
        if c == '\n' {
            self.line += 1;
            self.col = 0;
        }

        Some(c)
    }

    pub fn advance_by(&mut self, n: usize) -> Result<(), usize> {
        for i in 0..n {
            if self.next() == None {
                return Err(i);
            }
        }

        Ok(())
    }

    pub fn next_until(&mut self, test: fn(c: char) -> bool) -> String {
        let mut s = String::new();

        while let Some(c) = self.peek() {
            if test(c) {
                break;
            } else {
                s.push(c);
                self.next();
            }
        }

        s
    }

    /// Return a string of all characters since the last time this method was called.
    pub fn chop(&mut self) -> String {
        // TODO: find out how moving works ffs
        let tmp = self.chop_start.clone();
        self.chop_start = String::new();
        tmp
    }

    pub fn peek(&self) -> Option<char> {
        self.nth(0)
    }

    pub fn nth(&self, n: usize) -> Option<char> {
        self.chars().nth(n)
    }

    pub fn line(&self) -> usize {
        self.line
    }

    pub fn col(&self) -> usize {
        self.col
    }

    pub fn chars(&self) -> Chars<'a> {
        self.stream.clone()
    }
}

// Copyright (C) 2021 Fefie <owo@fef.moe>
// This file is part of gayconf.
//
// gayconf is non-violent software: you can use, redistribute,
// and/or modify it under the terms of the CNPLv6+ as found
// in the LICENSE file in the source code root directory or
// at <https://git.pixie.town/thufie/CNPL>.
//
// gayconf comes with ABSOLUTELY NO WARRANTY, to the extent
// permitted by applicable law.  See the CNPL for details.
